(function(window, undefined) {
	'use strict';
	var document = window.document,
	isScriptOnload = !!document.addEventListener, modClassName,
	rProtocol = /^(http(?:s)?\:\/\/|file\:.+\:\/)/, rModId = /([^\/?]+?)(\.(?:js|css))?(\?.*)?$/, rReadyState = /loaded|complete|undefined/,
	moduleCache = {},
	modifyCache = {}, 
	moduleOptions = {
		baseUrl : null,
		charset : {}
	},
	head = document.head || 
	document.getElementsByTagName( 'head' )[0] || 
	document.documentElement,
	seed={};
	seed.mix = function(target, source, override, whitelist) {
		if (!target || !source)
			return;
		if (override === undefined) {
			override = true;
		}
		var prop, len, i, _mix = function(prop) {
			if (override === true || !(prop in target)) {
				target[prop] = source[prop];
			}
		};
		if (whitelist && (len = whitelist.length)) {
			for (i = len; i;) {
				prop = whitelist[--i];
				if (prop in source) {
					_mix(prop);
				}
			}
		} else {
			for (prop in source) {
				_mix(prop);
			}
		}
		return target;
	};
	seed.mix(seed,{
		__uuid__ : 2,
		module : {},
		merge : function() {
			var result = {};
			for ( var i = 0, len = arguments.length; i < len; i++) {
				seed.mix(result, arguments[i]);
			}

			return result;
		},
		guid : function(pre) {
			return (pre || 'seed_') + (+new Date())
			+ (Math.random() + '').slice(-8);
		},
		modify : function(name, fn) {
			modifyCache[name] = fn;
		},
		config : function(options) {
			var baseUrl = options.baseUrl, isHttp = baseUrl
			.slice(0, 4) === 'http';
			if (isHttp) {
				moduleOptions.baseUrl = baseUrl;
			}
			else {
				moduleOptions.baseUrl = Module.mergePath(
					baseUrl, document.location.href);
			}
			moduleOptions.charset = seed.merge(
				moduleOptions.charset, options.charset);
		},
		error : function(msg) {
			throw new Error(msg);
		}
	});
	seed.euid = seed.guid();
	modClassName = seed.guid('easyJS_mod_');
	var Module = {
		init : function() {
			var i = 0, script, scripts, initMod, url;
			if (document.currentScript) {
				script = document.currentScript;
			} else {
				scripts = document.getElementsByTagName('script');
				script = scripts[scripts.length - 1];
			}
			initMod = script.getAttribute('data-main');
			url = script.hasAttribute ? script.src : script.getAttribute('src',
				4);
			moduleOptions.baseUrl = url.slice(0, url.lastIndexOf('/') + 1);
			if (initMod) {
				initMod = initMod.split(',');
				seed.use(initMod);
			}
			scripts = script = null;
		},
		getCurrentScript : function() {
			var script, scripts, i, stack;
			try {
				seed[seed.euid]();
			}
			catch (e) {
				stack = e.stack;
			}
			if (stack) {
				stack = stack.split(/[@ ]/g).pop();
				stack = stack[0] === '(' ? stack.slice(1, -1) : stack.replace(
					/\s/, '');
				return stack.replace(/(:\d+)?:\d+$/i, '').match(rModId)[1];
			}
			scripts = head.getElementsByTagName('script');
			i = scripts.length - 1;
			for (; i >= 0; i--) {
				script = scripts[i];
				if (script.className === modClassName
					&& script.readyState === 'interactive') {
					break;
			}
		}
		return script.src.match(rModId)[1];
	},
	mergePath : function(id, url) {
		var isHttp = url.slice(0, 4) === 'http', domain = '', i = 0, protocol, isHttp, urlDir, idDir, dirPath, len, dir;
			//alert(url);
			protocol = url.match(rProtocol)[1];
			url = url.slice(protocol.length);
			if (isHttp) {
				domain = url.slice(0, url.indexOf('/') + 1);
				url = url.slice(domain.length);
			}
			urlDir = url.split('/');
			urlDir.pop();
			idDir = id.split('/');
			idDir.pop();
			len = idDir.length;
			for (; i < len; i++) {
				dir = idDir[i];
				if (dir === '..') {
					urlDir.pop();
				} else if (dir !== '.') {
					urlDir.push(dir);
				}
			}
			dirPath = urlDir.join('/');
			dirPath = dirPath === '' ? '' : dirPath + '/';
			return protocol + domain + dirPath;
		},
		parseModId : function(id, url) {
			var isAbsoluteId = rProtocol.test(id), result = id.match(rModId), modName = result[1], suffix = result[2]
			|| '.js', search = result[3] || '', baseUrl, modUrl;
			if (isAbsoluteId) {
				url = id;
				id = '';
			}
			baseUrl = Module.mergePath(id, url);
			modUrl = baseUrl + modName + suffix + search;
			return [ modName, modUrl ];
		},
		getExports : function(deps) {
			if (deps) {
				var len = deps.length, module = seed.module, arr = [], i = 0, j = 0, dep;
				for (; i < len; i++) {
					arr[j++] = module[deps[i]].exports;
				}
				return arr;
			}
			return [];
		},
		isLoaded : function(mod) {
			var deps = mod.deps, len = deps.length, module = seed.module, i = 0, depMod;
			for (; i < len; i++) {
				depMod = module[deps[i]];
				if (depMod.status !== 4) {
					return false;
				}
			}
			return true;
		},
		factoryHandle : function(name, mod, factory, data) {
			mod.status = 3;
			var args = Module.getExports(mod.deps), exports = typeof factory === 'function' ? factory
			.apply(null, args)
			: factory;
			if (exports !== undefined) {
				if (modifyCache[name]) {
					exports = modifyCache[name](exports);
					delete modifyCache[name];
				}
				mod.exports = exports;
			}
			mod.status = 4;
			if (data) {
				data.length--;
			}
		},
		fireFactory : function(useKey) {
			var data = moduleCache[useKey], factorys = data.factorys, result = factorys[0], args, exports, name, toDepMod;
			if (!result) {
				return;
			}
			name = result.name;
			toDepMod = seed.module[name];
			if (Module.isLoaded(toDepMod)) {
				factorys.shift();
				Module.factoryHandle(name, toDepMod, result.factory, data);
				if (factorys.length) {
					Module.fireFactory(useKey);
				}
			}
		},
		complete : function(mod) {
			var module = seed.module, useKey = mod.useKey, data = moduleCache[useKey], i = 0, j = 0, namesCache, args, len, cacheMod, name;
			delete mod.useKey;
			if (!data) {
				return;
			}
			if (data.urls.length) {
				Module.load(useKey);
			} else if (!data.length) {
				namesCache = data.namesCache;
				len = namesCache.length;
				args = [];
				for (; i < len; i++) {
					name = namesCache[i];
					cacheMod = module[name];
					if (cacheMod.status !== 4) {
						seed.error('[' + name + '] module failed to use.');
					}
					args[j++] = cacheMod.exports;
				}
				if (data.callback) {
					data.callback.apply(null, args);
				}
				delete moduleCache[useKey];
			}
		},
		create : function(url, name, useKey) {
			var charset = moduleOptions.charset[name], mod = seed.module[name], script, link;
			mod.useKey = useKey;
			mod.status = 1;
			if (~url.indexOf('.css')) {
				link = document.createElement('link');
				link.rel = 'stylesheet';
				link.href = url;
				if (charset) {
					link.charset = charset;
				}
				link.onload = link.onerror = function() {
					link = link.onload = link.onerror = null;
					mod.status = 4;
					moduleCache[useKey].length--;
					Module.fireFactory(useKey);
					Module.complete(mod);
				};
				return link;
			}
			script = document.createElement('script');
			script.className = modClassName;
			script.async = true;
			if (charset) {
				script.charset = charset;
			}
			if (isScriptOnload) {
				script.onerror = function() {
					script.onerror = script.onload = null;
					head.removeChild(script);
					script = null;
					seed.error('[' + name
						+ '] module failed to load, the url is ' + url
						+ '.');
				};
			}
			script[isScriptOnload ? 'onload' : 'onreadystatechange'] = function() {
				if (isScriptOnload || rReadyState.test(script.readyState)) {
					script[isScriptOnload ? 'onload' : 'onreadystatechange'] = null;
					head.removeChild(script);
					script = null;
					Module.complete(mod);
				}
			};
			script.src = url;
			return script;
		},
		load : function(useKey) {
			var data = moduleCache[useKey], names = data.names.shift(), urls = data.urls
			.shift(), len = urls.length, i = 0, script;
			for (; i < len; i++) {
				script = Module.create(urls[i], names[i], useKey);
				head.insertBefore(script, head.firstChild);
			}
		}
	};
	window.define = function(name, deps, factory) {
		if (typeof name !== 'string') {
			if (typeof name === 'function') {
				factory = name;
			} else {
				factory = deps;
				deps = name;
			}
			name = Module.getCurrentScript();
		} else if (deps !== undefined && factory === undefined) {
			factory = deps;
			deps = null;
		}
		var module = seed.module, mod = module[name], isRepeat = false, isLoaded = true, names = [], urls = [], insertIndex = 0, pullIndex = 0, useKey, data, modUrl, factorys, baseUrl, depMod, depName, result, exports, args, depsData, repeatDepsData, i, repeatName;
		if (!mod) {
			mod = module[name] = {};
			if (deps) {
				mod.deps = deps;
			}
			Module.factoryHandle(name, mod, factory);
			return;
		}
		useKey = mod.useKey;
		data = moduleCache[useKey];
		modUrl = mod.url;
		mod.status = 2;
		mod.deps = [];
		if (deps && deps.length) {
			baseUrl = modUrl.slice(0, modUrl.lastIndexOf('/') + 1);
			factorys = data.factorys;
			depsData = data.deps[name] = {};
			for (i = 0; i < deps.length; i++) {
				result = Module.parseModId(deps[i], baseUrl);
				depName = result[0];
				depMod = module[depName];
				mod.deps.push(depName);
				depsData[depName] = true;
				if (depMod) {
					if (depMod.status !== 4) {
						if (!isRepeat) {
							isRepeat = true;
							repeatName = depName;
						}
						isLoaded = false;
					}
					deps.splice(i--, 1);
					continue;
				} else {
					depMod = module[depName] = {};
				}
				isLoaded = false;
				data.length++;
				names[names.length++] = depName;
				urls[urls.length++] = depMod.url = result[1];
			}
			if (!isLoaded) {
				factorys.unshift({
					name : name,
					factory : factory
				});
				if (repeatName) {
					repeatDepsData = data.deps[repeatName];
					for (i = factorys.length - 1; i >= 0; i--) {
						result = factorys[i].name;
						if (result === repeatName) {
							pullIndex = i;
							if (!repeatDepsData) {
								break;
							}
						}
						if (repeatDepsData && repeatDepsData[result]) {
							insertIndex = i;
							break;
						}
					}
					factorys.splice(insertIndex + 1, 0, factorys.splice(
						pullIndex, 1)[0]);
					factorys.splice(insertIndex + 1, 0, factorys.shift());
				}
			}
			if (names.length) {
				data.names.unshift(names);
				data.urls.unshift(urls);
			}
		}
		if (isLoaded) {
			Module.factoryHandle(name, mod, factory, data);
		}
		Module.fireFactory(useKey);
		if (!mod.deps.length) {
			delete mod.deps;
		}
	};
	window.require = function(ids, fn) {
		ids = typeof ids === 'string' ? [ ids ] : ids;
		var module = seed.module, len = ids.length, isLoaded = true, namesCache = [], modNames = [], modUrls = [], j = 0, mod, modName, result, useKey, args, name, i;
		for (i = 0; i < len; i++) {
			result = Module.parseModId(ids[i], moduleOptions.baseUrl);
			modName = result[0];
			mod = module[modName];
			if (!mod) {
				mod = module[modName] = {};
				isLoaded = false;
			}
			modNames[modNames.length++] = modName;
			modUrls[modUrls.length++] = mod.url = result[1];
		}
		useKey = modNames.join('_') + '_' + seed.guid();
		namesCache = namesCache.concat(modNames);
		if (isLoaded) {
			len = namesCache.length;
			args = [];
			for (i = 0; i < len; i++) {
				name = namesCache[i];
				mod = module[name];
				if (mod.status !== 4) {
					seed.error('[' + name + '] module failed to use.');
				}
				args[j++] = mod.exports;
			}
			if (fn) {
				fn.apply(null, args);
			}
			return;
		}
		moduleCache[useKey] = {
			length : namesCache.length,
			namesCache : namesCache,
			names : [ modNames ],
			urls : [ modUrls ],
			callback : fn,
			factorys : [],
			deps : {}
		};
		Module.load(useKey);
	};
	Module.init();
})(window);